/* libsocket++ - Socket abstraction library for C++
 *  Copyright (C) 2002  Simon Richter
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: builddSocketTCP.cc,v 1.2 2002/04/15 17:14:18 geier Exp $
 */

#include "SocketTCP.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <cstdio>

namespace socketxx {

static int makeSocket(void);

SocketTCP::SocketTCP(void) : Socket(makeSocket()) {
	return;
}

static int makeSocket(void) {
	int nsock = socket(AF_INET, SOCK_STREAM, 0);
	if(nsock == -1)
		throw Socket::CannotCreateSocketException();
	return nsock;
}

SocketTCP::SocketTCP(int fd) : Socket(fd) {
	return;
}

SocketTCP::~SocketTCP(void) {
	return;
}

const std::string SocketTCP::TypeTCP::typetcpname("tcp");
SocketTCP::TypeTCP SocketTCP::TypeTCP::typetcp;

Socket *SocketTCP::accept(void) {
	int newfd = ::accept(getfd(), 0, 0);
	if(newfd == -1)
		throw CannotAcceptException();
	return new SocketTCP(newfd);
}

static void parseAddress(const std::string &, struct sockaddr_in &);

void SocketTCP::bind(const std::string &addr) {
	struct sockaddr_in sa;
	parseAddress(addr, sa);
	if(::bind(getfd(), (struct sockaddr *)&sa, sizeof(sa)) != 0)
		throw CannotBindException();
	return;
}

void SocketTCP::connect(const std::string &addr) {
	struct sockaddr_in sa;
	parseAddress(addr, sa);
	if(::connect(getfd(), (struct sockaddr *)&sa, sizeof(sa)) != 0)
		throw CannotConnectException();
	return;
}

static void parseAddress(const std::string &addr, struct sockaddr_in &sa) {
	// Format is either host:port or just a port number
	sa.sin_family = AF_INET;
	std::string::size_type pos = addr.find(':');
	if(pos == std::string::npos) {
		sa.sin_addr.s_addr = INADDR_ANY;
		unsigned short port;
		if(::sscanf(addr.c_str(), "%hu", &port) != 1)
			throw Socket::InvalidAddressException();
		sa.sin_port = htons(port);
	} else {
		std::string hostaddr(addr, 0, pos);
		std::string portaddr(addr, pos + 1);
		if(inet_aton(hostaddr.c_str(), &sa.sin_addr) == 0)
			throw Socket::InvalidAddressException();
		unsigned short port;
		if(::sscanf(portaddr.c_str(), "%hu", &port) != 1)
			throw Socket::InvalidAddressException();
		sa.sin_port = htons(port);
	}
	return;
}

SocketTCP::TypeTCP::TypeTCP(void) {
	return;
}

SocketTCP::TypeTCP::~TypeTCP(void) {
	return;
}

const std::string &SocketTCP::TypeTCP::getType(void) const {
	return typetcpname;
}

Socket *SocketTCP::TypeTCP::operator()(void) const {
	return new SocketTCP();
}

}
