/* libsocket++ - Socket abstraction library for C++
 *  Copyright (C) 2002  Simon Richter
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation; either version 2.1 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: builddSocketUnix.cc,v 1.2 2002/04/15 17:14:19 geier Exp $
 */

#include "SocketUnix.h"

#include <sys/socket.h>
#include <sys/un.h>

namespace socketxx {

static int makeSocket(void);

SocketUnix::SocketUnix(void) : Socket(makeSocket()) {
	return;
}

static int makeSocket(void) {
	int nsock = socket(AF_UNIX, SOCK_STREAM, 0);
	if(nsock == -1)
		throw Socket::CannotCreateSocketException();
	return nsock;
}

SocketUnix::SocketUnix(int fd) : Socket(fd) {
	return;
}

SocketUnix::~SocketUnix(void) {
	return;
}

const std::string SocketUnix::TypeUnix::typeunixname("unix");
SocketUnix::TypeUnix SocketUnix::TypeUnix::typeunix;

Socket *SocketUnix::accept(void) {
	int newfd = ::accept(getfd(), 0, 0);
	if(newfd == -1)
		throw CannotAcceptException();
	return new SocketUnix(newfd);
}

static void parseAddress(const std::string &, struct sockaddr_un &);

void SocketUnix::bind(const std::string &addr) {
	struct sockaddr_un sa;
	parseAddress(addr, sa);
	if(::bind(getfd(), (struct sockaddr *)&sa, sizeof(sa)) != 0)
		throw CannotBindException();
	return;
}

void SocketUnix::connect(const std::string &addr) {
	struct sockaddr_un sa;
	parseAddress(addr, sa);
	if(::connect(getfd(), (struct sockaddr *)&sa, sizeof(sa)) != 0)
		throw CannotConnectException();
	return;
}

static void parseAddress(const std::string &addr, struct sockaddr_un &sa) {
	sa.sun_family = AF_UNIX;
	std::string::size_type len = addr.length();
	if(len >= sizeof(sa.sun_path))
		throw Socket::InvalidAddressException();
	addr.copy(sa.sun_path, sizeof(sa.sun_path));
	sa.sun_path[len] = '\0';
	return;
}

SocketUnix::TypeUnix::TypeUnix(void) {
	return;
}

SocketUnix::TypeUnix::~TypeUnix(void) {
	return;
}

const std::string &SocketUnix::TypeUnix::getType(void) const {
	return typeunixname;
}

Socket *SocketUnix::TypeUnix::operator()(void) const {
	return new SocketUnix();
}

}
